[FFmpeg-devel] [PATCH 1/1] Add vpx range encoder support

MihirGore mihirmvgore at gmail.com
Wed Mar 5 18:29:34 EET 2025


From: MihirGore23 <mihirvgore at outlook.com>

---
 libavcodec/vpx_rac.h | 75 ++++++++++++++++++++++++++++++++++----------
 1 file changed, 59 insertions(+), 16 deletions(-)

diff --git a/libavcodec/vpx_rac.h b/libavcodec/vpx_rac.h
index b158cc0754..f01358f71f 100644
--- a/libavcodec/vpx_rac.h
+++ b/libavcodec/vpx_rac.h
@@ -20,7 +20,7 @@
 
 /**
  * @file
  * Common VP5-VP9 range decoder stuff
  */
 
 #ifndef AVCODEC_VPX_RAC_H
@@ -34,24 +34,25 @@
 
 typedef struct VPXRangeCoder {
     int high;
-    int bits; /* stored negated (i.e. negative "bits" is a positive number of
-                 bits left) in order to eliminate a negate in cache refilling */
+    int bits; 
     const uint8_t *buffer;
     const uint8_t *end;
     unsigned int code_word;
     int end_reached;
+    uint8_t *output_buffer; // Added for encoding
+    uint8_t *output_end;    // Added for encoding
 } VPXRangeCoder;
 
 extern const uint8_t ff_vpx_norm_shift[256];
+
+/*Decoder Functions */
+
 int ff_vpx_init_range_decoder(VPXRangeCoder *c, const uint8_t *buf, int buf_size);
 
-/**
- * returns 1 if the end of the stream has been reached, 0 otherwise.
- */
 static av_always_inline int vpx_rac_is_end(VPXRangeCoder *c)
 {
     if (c->end <= c->buffer && c->bits >= 0)
-        c->end_reached ++;
+        c->end_reached++;
     return c->end_reached > 10;
 }
 
@@ -64,7 +65,7 @@ static av_always_inline unsigned int vpx_rac_renorm(VPXRangeCoder *c)
     c->high   <<= shift;
     code_word <<= shift;
     bits       += shift;
-    if(bits >= 0 && c->buffer < c->end) {
+    if (bits >= 0 && c->buffer < c->end) {
         code_word |= bytestream_get_be16(&c->buffer) << bits;
         bits -= 16;
     }
@@ -72,12 +73,6 @@ static av_always_inline unsigned int vpx_rac_renorm(VPXRangeCoder *c)
     return code_word;
 }
 
-#if   ARCH_ARM
-#include "arm/vpx_arith.h"
-#elif ARCH_X86
-#include "x86/vpx_arith.h"
-#endif
-
 #ifndef vpx_rac_get_prob
 #define vpx_rac_get_prob vpx_rac_get_prob
 static av_always_inline int vpx_rac_get_prob(VPXRangeCoder *c, uint8_t prob)
@@ -95,7 +90,6 @@ static av_always_inline int vpx_rac_get_prob(VPXRangeCoder *c, uint8_t prob)
 #endif
 
 #ifndef vpx_rac_get_prob_branchy
-// branchy variant, to be used where there's a branch based on the bit decoded
 static av_always_inline int vpx_rac_get_prob_branchy(VPXRangeCoder *c, int prob)
 {
     unsigned long code_word = vpx_rac_renorm(c);
@@ -117,7 +111,6 @@ static av_always_inline int vpx_rac_get_prob_branchy(VPXRangeCoder *c, int prob)
 static av_always_inline int vpx_rac_get(VPXRangeCoder *c)
 {
     unsigned int code_word = vpx_rac_renorm(c);
-    /* equiprobable */
     int low = (c->high + 1) >> 1;
     unsigned int low_shift = low << 16;
     int bit = code_word >= low_shift;
@@ -132,4 +125,54 @@ static av_always_inline int vpx_rac_get(VPXRangeCoder *c)
     return bit;
 }
 
+//Encoder Functions
+
+int ff_vpx_init_range_encoder(VPXRangeCoder *c, uint8_t *buf, int buf_size);
+
+static av_always_inline void vpx_rac_renorm_encoder(VPXRangeCoder *c)
+{
+    while (c->high < 0x10000) {
+        if (c->output_buffer < c->output_end) {
+            *c->output_buffer++ = c->code_word >> 24;
+        }
+        c->code_word <<= 8;
+        c->high <<= 8;
+    }
+}
+
+static av_always_inline void vpx_rac_put_prob(VPXRangeCoder *c, int bit, uint8_t prob)
+{
+    unsigned int low = 1 + (((c->high - 1) * prob) >> 8);
+    if (bit) {
+        c->code_word += low << 16;
+        c->high -= low;
+    } else {
+        c->high = low;
+    }
+    vpx_rac_renorm_encoder(c);
+}
+
+static av_always_inline void vpx_rac_put(VPXRangeCoder *c, int bit)
+{
+    unsigned int low = (c->high + 1) >> 1;
+    if (bit) {
+        c->code_word += low << 16;
+        c->high -= low;
+    } else {
+        c->high = low;
+    }
+    vpx_rac_renorm_encoder(c);
+}
+
+static av_always_inline void vpx_rac_flush(VPXRangeCoder *c)
+{
+    for (int i = 0; i < 4; i++) {
+        if (c->output_buffer < c->output_end) {
+            *c->output_buffer++ = c->code_word >> 24;
+        }
+        c->code_word <<= 8;
+    }
+}
+
 #endif /* AVCODEC_VPX_RAC_H */
+
@@ -20,7 +20,7 @@
 /**
  * @file
- * Common VP5-VP9 range decoder stuff
+ * Common VP5-VP9 range encoder and decoder functions
  */
 
-- 
2.43.0



More information about the ffmpeg-devel mailing list